//
//  PhidgetAccelerometerManager.m
//  PhidgetAccelerometerPatch
//
//  Created by KIMURA Hiroaki on 06/08/23.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import "PhidgetAccelerometerManager.h"


@implementation PhidgetAccelerometerManager

static PhidgetAccelerometerManager* manager = nil;


/*
 * gotAttach and gotDetach just send notifications which are picked up by the proper 
 * GUI object method so they can access the GUI instance variables
 */
int gotAttach(CPhidgetManagerHandle phidm, void * ptr, CPhidgetHandle phid) {
	PhidgetInfo *temp = [PhidgetInfo alloc];
	[temp setPhidget:phid];
	[[NSNotificationCenter defaultCenter] postNotificationName:@"gotAttach" object:temp];
	return 0;
}

int gotDetach(CPhidgetManagerHandle phidm, void * ptr, CPhidgetHandle phid) {
	PhidgetInfo *temp = [PhidgetInfo alloc];
	[temp setPhidget:phid];
	[[NSNotificationCenter defaultCenter] postNotificationName:@"gotDetach" object:temp];
	return 0;
}

int gotAccel(CPhidgetAccelerometerHandle phid, void *meh, int ind, double val) {
	AccelInfo *temp = [AccelInfo alloc];
	[temp setIndex:ind];
	[temp setVal:val];
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	//[[NSNotificationCenter defaultCenter] postNotificationName:@"accel" object:temp];
	long serialNum;
	CPhidget_getSerialNumber(phid, &serialNum);
	[[NSNotificationCenter defaultCenter] postNotificationName:[NSString stringWithFormat:@"accel%d", serialNum] object:temp];
	
	[pool release];
	return 0;
}



-(id)init{
	
	[self doesNotRecognizeSelector:_cmd];

}

+(PhidgetAccelerometerManager*)phidgetAccelerometerManager{
	if (!manager){
		manager = [[PhidgetAccelerometerManager alloc] initManager];
	}
	
	return manager;
}

- (id) initManager{
	
	handles = [[NSMutableDictionary alloc] initWithCapacity:10];
	
	
	CPhidgetManagerHandle phidlist;
	int something;
	
	
	/* add come notification listeners 
		* We need to do this so that the C functions that we hand over to the framework for notifications
		* can access the GUI indirectly by notifying an object method of the GUI
		*/
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(phidgetAdded:)
												 name:@"gotAttach" object:nil];
	
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(phidgetRemoved:)
												 name:@"gotDetach" object:nil];
	
	/*
	 * set up the phidget Manager - detects when phidgets are attached and removed
	 */
	CPhidgetManager_initialize(&phidlist);
	CPhidgetManager_set_AttachHandler(phidlist, &gotAttach, &something);
	CPhidgetManager_set_DetachHandler(phidlist, &gotDetach, &something);
	
	CPhidgetManager_setupNotifications(phidlist, CFRunLoopGetCurrent());
	
	
	
	//[NSThread detachNewThreadSelector:@selector(pollAccel:) toTarget:self withObject:nil];
	timer = [[NSTimer scheduledTimerWithTimeInterval:0.005
												  target:self
												selector:@selector(pollAccel:)
												userInfo:nil
												 repeats:YES]retain];
	
	return self;
}

- (void)dealloc{
	[timer release];
	[handles release];
	
	[super release];
}

-(NSMutableDictionary*) handles{
	return handles;
}


////////////////////////Phidgets///////////////////////////////

//this is called by the timer in the run loop
- (void)pollAccel:(id)sender;
{	
	
	//while(1) {
		NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

		NSEnumerator* enumerator = [handles objectEnumerator];
		NSNumber* num;
		
		while (num = [enumerator nextObject]){
			//NSLog(@"");
			long handle = [num longValue];
			CPhidgetAccelerometer_read(handle);
		}		

		[pool release];
	//}
	
}

/*
 * check if it's a Accelerometer controller, open it and enable the controls
 */
- (void)phidgetAdded:(NSNotification *)notification
{
    PhidgetInfo* phidgetInfo = [notification object];
	CPhidgetHandle phid = [phidgetInfo getPhidget];
	long serial;
	char *tempChar;
	int err;
	
	CPhidget_getDeviceType(phid,&tempChar);
	if(!strcmp(tempChar,"PhidgetAccelerometer")) {
		CPhidget_getSerialNumber(phid, &serial);
		CPhidgetAccelerometer_create((CPhidgetAccelerometerHandle*)&phid);
		
		err = CPhidget_open(phid, serial);
		if (err){
			NSLog(@"Phidget open error!. serial#: %d", serial);
			return;
		}else{
			NSNumber* num = [[NSNumber alloc] initWithLong:phid];
			[handles setObject:num forKey:[NSString stringWithFormat:@"%d", serial]];
			[num release];
			CPhidgetAccelerometer_set_OnAccelerationChange_Handler(phid, &gotAccel, NULL);
		}
	}
	
	[[NSNotificationCenter defaultCenter] postNotificationName:@"updateUI" object:nil];

}


/*
 * check if it's the accelerometer we've been controlling and if so disable the controls
 */
- (void)phidgetRemoved:(NSNotification *)notification
{
    PhidgetInfo* phidgetInfo = [notification object];
	CPhidgetHandle phid = [phidgetInfo getPhidget];
	long serial;
	char *tempChar;
	
	CPhidget_getDeviceType(phid,&tempChar);
	if(!strcmp(tempChar,"PhidgetAccelerometer")) {
		CPhidget_getSerialNumber(phid, &serial);
		
		[handles removeObjectForKey:[NSString stringWithFormat:@"%d", serial]];
		

		CPhidgetAccelerometer_close(phid);

	}
	
	[[NSNotificationCenter defaultCenter] postNotificationName:@"updateUI" object:nil];

}



@end
